require "prefabutil" --Used for the MakePlacer
---------------
local name = "korriganDS" 
local folder = KnownModIndex:GetModActualName(name)

local MUSHPERDAY=GetModConfigData("MUSHPERDAY", folder)	
local MAXMUSH=GetModConfigData("MAXMUSH", folder)
---------------
local childtype1="blue_mushroom"
local childtype2="red_mushroom"
local childtype3="green_mushroom"
---------------

local assets =
{
	Asset("ATLAS", "images/inventoryimages/mushroomfarm.xml"),
	Asset("IMAGE", "images/inventoryimages/mushroomfarm.tex"),
	Asset("ANIM", "anim/mushroomfarm.zip"),
}
local anim={}
anim.idle = "idle_mush"
anim.hit = anim.idle
anim.place ="idle"

local prefabs =
{
    childtype1,
	childtype2,
	childtype3,
	
    "collapse_small",
}



local function onhammered(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function onhit(inst, worker)
	if not inst:HasTag("burnt") then 
		inst.AnimState:PlayAnimation(inst.anims.hit)
		inst.AnimState:PushAnimation(inst.anims.idle, false)
	end
end

local function OnSave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
		data.burnt = true
	end
end

local function OnLoad(inst, data)
	if data and data.burnt then
		inst.components.burnable.onburnt(inst)
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation(anim.place)
	inst.AnimState:PushAnimation(anim.idle, false)
end

local function PlantMush(inst)
	local minrad=1.6
	local maxrad=3.5
	
	if not inst then return end
	local pt = inst:GetPosition()
	
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
	local count=0
	for k,v in pairs(ents) do
		if v.prefab == childtype1 then
			count = count + 1
		end
	end
    if count>=MAXMUSH then return end
	
	local angle=math.random()*2*PI
	local radius=math.random()*(maxrad-minrad)+minrad
	local offset, check_angle, deflected=FindWalkableOffset(pt, angle, radius, 8, true, false)
	if(not check_angle) then return end
	angle=check_angle
	pt.x=pt.x+radius*math.cos(angle)
	pt.z=pt.z-radius*math.sin(angle)
	
	SpawnPrefab(childtype1).Transform:SetPosition(pt:Get())

	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
	local count=0
	for k,v in pairs(ents) do
		if v.prefab == childtype2 then
			count = count + 1
		end
	end
	if count>=MAXMUSH then return end

	local angle=math.random()*2*PI
	local radius=math.random()*(maxrad-minrad)+minrad
	local offset, check_angle, deflected=FindWalkableOffset(pt, angle, radius, 9, true, false)
	if(not check_angle) then return end
	angle=check_angle
	pt.x=pt.x+radius*math.cos(angle)
	pt.z=pt.z-radius*math.sin(angle)
	
	SpawnPrefab(childtype2).Transform:SetPosition(pt:Get())

	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
	local count=0
	for k,v in pairs(ents) do
		if v.prefab == childtype3 then
			count = count + 1
		end
	end
	if count>=MAXMUSH then return end
	
	local angle=math.random()*2*PI
	local radius=math.random()*(maxrad-minrad)+minrad
	local offset, check_angle, deflected=FindWalkableOffset(pt, angle, radius, 7, true, false)
	if(not check_angle) then return end
	angle=check_angle
	pt.x=pt.x+radius*math.cos(angle)
	pt.z=pt.z-radius*math.sin(angle)
	
	SpawnPrefab(childtype3).Transform:SetPosition(pt:Get())
end

local function saveanims(inst)
	if inst.anims==nil then
		inst.anims = {}
	end
	
	inst.anims.idle = anim.idle
	inst.anims.hit = anim.hit
	inst.anims.place = anim.place
end

local function fn()

	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()	
	MakeObstaclePhysics(inst, 1)
	
	inst.MiniMapEntity:SetIcon("mushroomfarm.tex")
	inst.AnimState:SetBank("mushroomfarm")
	inst.AnimState:SetBuild("mushroomfarm")
	inst.AnimState:PlayAnimation(anim.idle)
	
	inst:AddTag("structure")
	inst:AddTag("playerowned")
	
	local growth_stages = {
		{name="only",
		time=function(inst) return TUNING.TOTAL_DAY_TIME/MUSHPERDAY end,
		fn=function(inst) PlantMush(inst) end},
	}
	inst:AddComponent("growable")
	inst.components.growable.stages = growth_stages
	inst.components.growable:SetStage(1)
	inst.components.growable.loopstages = true
	inst.components.growable:StartGrowing()
	
	inst:AddComponent("inspectable")
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)

	saveanims(inst)

	MakeSnowCovered(inst)
	inst:ListenForEvent("onbuilt", onbuilt)
	MakeSmallBurnable(inst)
    MakeSmallPropagator(inst)
   
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	return inst
end

return	Prefab("common/objects/mushroomfarm", fn, assets, prefabs),
MakePlacer("common/mushroomfarm_placer", "mushroomfarm", "mushroomfarm", "idle" )